
#include "sys.h"
#include <stdlib.h>
#include <string.h>

//[of]:outOfMemory
static void outOfMemory()
{
	exit(1);
}
//[cf]

#ifdef DEBUG
	// These global variables are public, so any program can report a leak
	static size_t totalBlock = 0;
	static size_t leftBlock = 0;
	static size_t leftBytes = 0;
	static size_t maxBytes = 0;
	
//[of]:	allocate
ENTRY void* csys_allocate (size_t size)
{
	char* p;
	
	if( size == 0 )
		return NULL;
	
	p = (char*)malloc(size + sizeof(int) * 2 + sizeof(size_t) );
	if( p == NULL )
		outOfMemory();
	
	// Statistics
	totalBlock ++;
	leftBlock ++;
	leftBytes += size;
	if( leftBytes > maxBytes )
		maxBytes = leftBytes;

	*(int*)p = 0x1234;
	*(size_t*)(p + sizeof(int)) = size;
	*(int*)(p + sizeof(int) + sizeof (size_t) + size) = 0x5678;

	return (void*)(p + sizeof(int) + sizeof(size_t));
}
//[cf]
//[of]:	free
ENTRY void csys_free (void *p, size_t size)
{
	char* q;
	
	// Freeing a null pointer is valid and has no effect
	if( p == 0 )
		return;
	
	// Statistics
	leftBytes -= size;
	leftBlock--;

	q = (char*)p - (sizeof(int) + sizeof(size_t));
	if( *(int*)q != 0x1234 )
	{
		// Invalid free
		csys_trap();
	}
	if( *(size_t*)(q + sizeof(int)) != size )
	{
		// Invalid free size
		csys_trap();
	}
	if( *(int*)(q + sizeof(int) + sizeof(size_t) + size) != 0x5678 )
	{
		// Heap corruption after block
		csys_trap();
	}
	*(int*)q = 0xCCC;

	memset(p, 0xE5, size);
	free(q);
}
//[cf]
//[of]:	getMemoryStatistics
ENTRY void csys_getMemoryStatistics(struct MemoryStatistics* stats)
{
	stats->totalBlock = totalBlock;
	stats->leftBlock = leftBlock;
	stats->leftBytes = leftBytes;
	stats->maxBytes = maxBytes;
}
//[cf]
#else

//[of]:	allocate
ENTRY void* csys_allocate (size_t size)
{
	void* p = malloc(size);
	if( p == NULL && size != 0 )
		outOfMemory();
	return p;
}
//[cf]
//[of]:	free
ENTRY void csys_free (void* p, size_t size)
{
	free(p);
}
//[cf]
//[of]:	getMemoryStatistics
ENTRY void csys_getMemoryStatistics(struct MemoryStatistics* stats)
{
	stats->totalBlock = 0;
	stats->leftBlock = 0;
	stats->leftBytes = 0;
	stats->maxBytes = 0;
}
//[cf]
#endif
