;[of]:General
!include "MUI.nsh"

; Name used as key in registry
!define APP "Code Browser"

; The name of the installer
Name "Code Browser"

; The file to write
OutFile "${RELEASEDIR}\code-browser-${VERSION}-installer.exe"

; The default installation directory
InstallDir "$PROGRAMFILES\Code Browser"

; Registry key to check for directory (so if you install again, it will 
; overwrite the old one automatically)
InstallDirRegKey HKLM "Software\${APP}" "InstallDir"

;LicenseData COPYING

;[cf]
;[of]:Pages
;Page license
;Page components
;Page directory
;Page instfiles

;UninstPage uninstConfirm
;UninstPage instfiles

!define MUI_HEADERIMAGE
!define MUI_ABORTWARNING
!define MUI_FINISHPAGE_RUN "$INSTDIR\cb.exe"
!define MUI_FINISHPAGE_RUN_PARAMETERS "$\"$INSTDIR\examples\tutorial.txt$\""
!define MUI_FINISHPAGE_RUN_TEXT "Run Code Browser and Open Tutorial File"

!insertmacro MUI_PAGE_WELCOME
!insertmacro MUI_PAGE_LICENSE "COPYING.txt"
!insertmacro MUI_PAGE_COMPONENTS
!insertmacro MUI_PAGE_DIRECTORY
!insertmacro MUI_PAGE_INSTFILES
!insertmacro MUI_PAGE_FINISH

!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES

!insertmacro MUI_LANGUAGE "English"

;[cf]
;[of]:Section\: Main
Section "Code Browser"

  SectionIn RO

  ; Delete any previous version file
  RMDir /r $INSTDIR\docs
  RMDir /r $INSTDIR\samples
  RMDir /r $INSTDIR\config
  RMDir /r $INSTDIR\config-default
  RMDir /r $INSTDIR\config-optional

  ; Set output path to the installation directory.
  SetOutPath $INSTDIR

  File "cb.exe"
  File "COPYING.txt"
  File "README.txt"
  File "session.cbc"
  File /r "docs"
  File /r "examples"
  File /r "config"
  File /r "config-optional"

  ; Write the installation path into the registry
  WriteRegStr HKLM "SOFTWARE\${APP}" "InstallDir" "$INSTDIR"
  
  ; Write the uninstall keys for Windows
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${APP}" "DisplayName" "Code Browser"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${APP}" "UninstallString" '"$INSTDIR\uninstall.exe"'
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${APP}" "NoModify" 1
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${APP}" "NoRepair" 1
  WriteUninstaller "uninstall.exe"

SectionEnd
;[cf]
;[of]:Section\: Start Menu Shortcuts
;[c]Start Menu Shortcuts (can be disabled by the user)
;[c]
Section "Start Menu Shortcuts"

  CreateDirectory "$SMPROGRAMS\${APP}"
  CreateShortCut "$SMPROGRAMS\${APP}\Uninstall.lnk" "$INSTDIR\uninstall.exe" "" "$INSTDIR\uninstall.exe" 0
  CreateShortCut "$SMPROGRAMS\${APP}\Code Browser.lnk" "$INSTDIR\cb.exe" "" "$INSTDIR\cb.exe" 0
  CreateShortCut "$SMPROGRAMS\${APP}\Documentation.lnk" "$INSTDIR\docs\index.html" "" "$INSTDIR\docs\index.html" 0
  
SectionEnd
;[cf]
;[of]:Section\: Quick Launch
;[c]Quick Launch (can be disabled by the user)
;[c]
Section "Quick Launch"

  CreateShortCut "$QUICKLAUNCH\Code Browser.lnk" "$INSTDIR\cb.exe" "" "$INSTDIR\cb.exe" 0
  
SectionEnd
;[cf]
;[of]:Section\: Uninstall Section
Section "Uninstall"

  ; Remove registry keys
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${APP}"
  DeleteRegKey HKLM "SOFTWARE\${APP}"

  ; Remove files and uninstaller
  Delete $INSTDIR\uninstall.exe
  Delete $INSTDIR\cb.exe
  Delete $INSTDIR\COPYING.txt
  Delete $INSTDIR\README.txt
  Delete $INSTDIR\session.cbc
  RMDir /r $INSTDIR\docs
  RMDir /r $INSTDIR\examples
  RMDir /r $INSTDIR\config
  RMDir /r $INSTDIR\config-optional

  ; Remove shortcuts, if any
  Delete "$SMPROGRAMS\${APP}\*.*"
  Delete "$QUICKLAUNCH\Code Browser.lnk"

  ; Remove directories used
  RMDir "$SMPROGRAMS\${APP}"
  RMDir "$INSTDIR"

SectionEnd
;[cf]
